Type: Package
Package: teal
Title: Exploratory Web Apps for Analyzing Clinical Trials Data
Version: 1.2.1
Date: 2026-07-28
Authors@R: c(
    person("Dony", "Unardi", , "unardid@gene.com", role = c("aut", "cre")),
    person("Dawid", "Kaledkowski", , "dawid.kaledkowski@roche.com", role = "aut",
           comment = c(ORCID = "0000-0001-9533-457X")),
    person("Pawel", "Rucki", , "pawel.rucki@roche.com", role = "aut"),
    person("Aleksander", "Chlebowski", , "aleksander.chlebowski@contractors.roche.com", role = "aut",
           comment = c(ORCID = "0000-0001-5018-6294")),
    person("Andre", "Verissimo", , "andre.verissimo@roche.com", role = "aut",
           comment = c(ORCID = "0000-0002-2212-339X")),
    person("Kartikeya", "Kirar", , "kartikeya.kirar@businesspartner.roche.com", role = "aut"),
    person("Vedha", "Viyash", , "vedha.viyash@roche.com", role = "aut"),
    person("Marcin", "Kosinski", , "marcin.kosinski.mk1@roche.com", role = "aut"),
    person("Adrian", "Waddell", , "adrian.waddell@gene.com", role = "aut"),
    person("Nina", "Qi", , "qi.ting@gene.com", role = "rev"),
    person("Nikolas", "Burkoff", role = "aut"),
    person("Mahmoud", "Hallal", role = "aut"),
    person("Maciej", "Nasinski", role = "aut"),
    person("Konrad", "Pagacz", role = "aut"),
    person("Junlue", "Zhao", role = "aut"),
    person("Tadeusz", "Lewandowski", role = "aut"),
    person("Chendi", "Liao", role = "rev"),
    person("F. Hoffmann-La Roche AG", role = c("cph", "fnd")),
    person("Maximilian", "Mordig", role = "ctb")
  )
Description: A 'shiny' based interactive exploration framework for
    analyzing clinical trials data. 'teal' currently provides a dynamic
    filtering facility and different data viewers. 'teal' 'shiny'
    applications are built using standard 'shiny' modules.
License: Apache License 2.0
URL: https://insightsengineering.github.io/teal/,
        https://github.com/insightsengineering/teal/
BugReports: https://github.com/insightsengineering/teal/issues
Depends: R (>= 4.1), shiny (>= 1.8.1), teal.data (>= 0.8.0), teal.slice
        (>= 0.7.0)
Imports: bsicons, bslib (>= 0.8.0), checkmate (>= 2.1.0), cli,
        htmltools, jsonlite, lifecycle (>= 0.2.0), logger (>= 0.2.0),
        methods, rlang (>= 1.0.0), shinyjs, stats, teal.code (>=
        0.7.0), teal.logger (>= 0.4.0), teal.reporter (>= 0.6.0),
        teal.widgets (>= 0.5.0), tools, utils
Suggests: ggplot2 (>= 3.4.0), knitr (>= 1.42), mirai (>= 1.1.1),
        MultiAssayExperiment, R6, renv (>= 1.0.11), rmarkdown (>=
        2.23), roxy.shinylive, rvest (>= 1.0.0), shinytest2,
        shinyvalidate, testthat (>= 3.2.0), withr (>= 2.1.0), yaml (>=
        1.1.0)
VignetteBuilder: knitr, rmarkdown
RdMacros: lifecycle
Config/Needs/verdepcheck: rstudio/shiny, insightsengineering/teal.data,
        insightsengineering/teal.slice, mllg/checkmate,
        jeroen/jsonlite, r-lib/lifecycle, daroczig/logger, r-lib/mirai,
        r-lib/cli, r-lib/nanonext, rstudio/renv, r-lib/rlang,
        daattali/shinyjs, insightsengineering/teal.code,
        insightsengineering/teal.logger,
        insightsengineering/teal.reporter,
        insightsengineering/teal.widgets, rstudio/bslib, yihui/knitr,
        bioc::MultiAssayExperiment, r-lib/R6, rstudio/rmarkdown,
        tidyverse/rvest, rstudio/shinytest2, rstudio/shinyvalidate,
        r-lib/testthat, r-lib/withr, yaml=vubiostat/r-yaml,
        rstudio/htmltools, bioc::matrixStats,
        insightsengineering/roxy.shinylive
Config/Needs/website: insightsengineering/nesttemplate
Config/roxygen2/version: 8.0.0
Encoding: UTF-8
Language: en-US
Collate: 'TealAppDriver.R' 'after.R' 'checkmate.R'
        'decorators-helpers.R' 'dummy_functions.R' 'include_css_js.R'
        'modules.R' 'init.R' 'module_bookmark_manager.R'
        'module_data_summary.R' 'module_filter_data.R'
        'module_filter_manager.R' 'module_nested_tabs.R'
        'module_session_info.R' 'module_snapshot_manager.R'
        'module_source_code.R' 'module_teal.R' 'module_teal_lockfile.R'
        'module_teal_reporter.R' 'module_transform_data.R'
        'module_validate_error.R' 'reporter_previewer_module.R'
        'teal.R' 'teal_data_module.R' 'teal_data_module-eval_code.R'
        'teal_data_module-within.R' 'teal_data_utils.R'
        'teal_modifiers.R' 'teal_slices-store.R' 'teal_slices.R'
        'teal_transform_module.R' 'utils.R' 'validate_inputs.R'
        'validations.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2026-07-29 03:51:33 UTC; unardid
Author: Dony Unardi [aut, cre],
  Dawid Kaledkowski [aut] (ORCID:
    <https://orcid.org/0000-0001-9533-457X>),
  Pawel Rucki [aut],
  Aleksander Chlebowski [aut] (ORCID:
    <https://orcid.org/0000-0001-5018-6294>),
  Andre Verissimo [aut] (ORCID: <https://orcid.org/0000-0002-2212-339X>),
  Kartikeya Kirar [aut],
  Vedha Viyash [aut],
  Marcin Kosinski [aut],
  Adrian Waddell [aut],
  Nina Qi [rev],
  Nikolas Burkoff [aut],
  Mahmoud Hallal [aut],
  Maciej Nasinski [aut],
  Konrad Pagacz [aut],
  Junlue Zhao [aut],
  Tadeusz Lewandowski [aut],
  Chendi Liao [rev],
  F. Hoffmann-La Roche AG [cph, fnd],
  Maximilian Mordig [ctb]
Maintainer: Dony Unardi <unardid@gene.com>
Repository: CRAN
Date/Publication: 2026-07-29 16:30:02 UTC
Built: R 4.5.2; ; 2026-07-29 18:00:34 UTC; unix
